@@ -3,9 +3,18 @@
 /** @var \Icinga\Application\Modules\Module $this */
 
 /** @var \Icinga\Application\Modules\MenuItemContainer $section */
+
+use Icinga\Module\Graphite\ProvidedHook\Icingadb\IcingadbSupport;
+
 $section = $this->menuSection(N_('Graphite'), ['icon' => 'chart-area']);
-$section->add(N_('Hosts'), ['url' => 'graphite/list/hosts?graphs_limit=2']);
-$section->add(N_('Services'), ['url' => 'graphite/list/services?graphs_limit=2']);
+
+if ($this::exists('icingadb') && IcingadbSupport::useIcingaDbAsBackend()) {
+    $section->add(N_('Hosts'), ['url' => 'graphite/hosts']);
+    $section->add(N_('Services'), ['url' => 'graphite/services']);
+} else {
+    $section->add(N_('Hosts'), ['url' => 'graphite/list/hosts']);
+    $section->add(N_('Services'), ['url' => 'graphite/list/services']);
+}
 
 $this->provideConfigTab('backend', array(
     'title' => $this->translate('Configure the Graphite Web backend'),
