@@ -22,8 +22,7 @@ class DetailviewExtension extends DetailviewExtensionHook
         $graphs = (string) Graphs::forMonitoredObject($object)
             ->setWidth(440)
             ->setHeight(220)
-            ->setClasses(['monitored-object-detail-view'])
-            ->setMaxVisibleGraphs(2)
+            ->setClasses(['object-detail-view'])
             ->setPreloadDummy()
             ->setShowNoGraphsFound(false)
             ->handleRequest();
@@ -31,7 +30,9 @@ class DetailviewExtension extends DetailviewExtensionHook
         if ($graphs !== '') {
             $this->handleTimeRangePickerRequest();
             return '<h2>' . mt('graphite', 'Graphs') . '</h2>'
-                . $this->renderTimeRangePicker($this->getView()) . $graphs;
+                . $this->renderTimeRangePicker($this->getView())
+                . '<div class="graphite-graph-color-registry"></div>'
+                . $graphs;
         }
 
         return '';
