@@ -6,10 +6,11 @@ use Icinga\Application\Config;
 use Icinga\Application\Icinga;
 use Icinga\Authentication\Auth;
 use Icinga\Exception\ConfigurationError;
-use Icinga\Module\Graphite\Forms\TimeRangePicker\TimeRangePickerTrait;
+use Icinga\Module\Graphite\Graphing\Chart;
 use Icinga\Module\Graphite\Graphing\GraphingTrait;
 use Icinga\Module\Graphite\Graphing\Template;
 use Icinga\Module\Graphite\Util\InternalProcessTracker as IPT;
+use Icinga\Module\Graphite\Util\TimeRangePickerTools;
 use Icinga\Module\Graphite\Web\Widget\Graphs\Host as HostGraphs;
 use Icinga\Module\Graphite\Web\Widget\Graphs\Service as ServiceGraphs;
 use Icinga\Module\Monitoring\Object\Host;
@@ -19,31 +20,35 @@ use Icinga\Web\Request;
 use Icinga\Web\Url;
 use Icinga\Web\View;
 use Icinga\Web\Widget\AbstractWidget;
+use ipl\Orm\Model;
+use Icinga\Module\Icingadb\Model\Host as IcingadbHost;
+use Icinga\Module\Graphite\Web\Widget\Graphs\Icingadb\IcingadbHost as IcingadbHostGraphs;
+use Icinga\Module\Graphite\Web\Widget\Graphs\Icingadb\IcingadbService as IcingadbServiceGraphs;
 
 abstract class Graphs extends AbstractWidget
 {
     use GraphingTrait;
 
     /**
-     * The Icinga custom variable with the "real" check command (if any) of monitored objects we display graphs for
+     * The Icinga custom variable with the "real" check command (if any) of objects we display graphs for
      *
      * @var string
      */
     protected static $obscuredCheckCommandCustomVar;
 
     /**
-     * The type of the monitored object to render the graphs of
+     * The type of the object to render the graphs for
      *
      * @var string
      */
-    protected $monitoredObjectType;
+    protected $objectType;
 
     /**
-     * The monitored object to render the graphs of
+     * The object to render the graphs for
      *
-     * @var MonitoredObject
+     * @var MonitoredObject|Model
      */
-    protected $monitoredObject;
+    protected $object;
 
     /**
      * Graph image width
@@ -103,13 +108,6 @@ abstract class Graphs extends AbstractWidget
      */
     protected $classes = [];
 
-    /**
-     * The amount of graphs to show
-     *
-     * @var int
-     */
-    protected $maxVisibleGraphs;
-
     /**
      * Whether to serve a transparent dummy image first and let the JS code load the actual graph
      *
@@ -117,6 +115,13 @@ abstract class Graphs extends AbstractWidget
      */
     protected $preloadDummy = false;
 
+    /**
+     * Whether to render the graphs inline
+     *
+     * @var bool
+     */
+    protected $renderInline;
+
     /**
      * Whether to explicitly display that no graphs were found
      *
@@ -125,7 +130,7 @@ abstract class Graphs extends AbstractWidget
     protected $showNoGraphsFound;
 
     /**
-     * Factory, based on the given object
+     * Factory, based on the given monitoring object
      *
      * @param   MonitoredObject $object
      *
@@ -144,6 +149,22 @@ abstract class Graphs extends AbstractWidget
         }
     }
 
+    /**
+     * Factory, based on the given icingadb object
+     *
+     * @param Model $object
+     *
+     * @return static
+     */
+    public static function forIcingadbObject(Model $object)
+    {
+        if ($object instanceof IcingadbHost) {
+            return new IcingadbHostGraphs($object);
+        }
+
+        return new IcingadbServiceGraphs($object);
+    }
+
     /**
      * Get the Icinga custom variable with the "real" check command (if any) of monitored objects we display graphs for
      *
@@ -162,15 +183,22 @@ abstract class Graphs extends AbstractWidget
     /**
      * Constructor
      *
-     * @param   MonitoredObject $monitoredObject    The monitored object to render the graphs of
+     * @param MonitoredObject|Model $object    The object to render the graphs for
      */
-    public function __construct(MonitoredObject $monitoredObject)
+    public function __construct($object)
     {
-        $this->monitoredObject = $monitoredObject;
-        $this->checkCommand = $monitoredObject->{"{$this->monitoredObjectType}_check_command"};
-        $this->obscuredCheckCommand = $monitoredObject->{
-            "_{$this->monitoredObjectType}_" . Graphs::getObscuredCheckCommandCustomVar()
-        };
+        $this->object = $object;
+        $this->renderInline = Url::fromRequest()->getParam('format') === 'pdf';
+
+        if ($object instanceof Model) {
+            $this->checkCommand = $object->checkcommand_name;
+            $this->obscuredCheckCommand = $object->vars[Graphs::getObscuredCheckCommandCustomVar()] ?? null;
+        } else {
+            $this->checkCommand = $object->{"{$this->objectType}_check_command"};
+            $this->obscuredCheckCommand = $object->{
+                "_{$this->objectType}_" . Graphs::getObscuredCheckCommandCustomVar()
+            };
+        }
     }
 
     /**
@@ -202,7 +230,7 @@ abstract class Graphs extends AbstractWidget
     protected function getGraphsList()
     {
         $result = []; // kind of string builder
-        $imageBaseUrl = $this->preloadDummy ? $this->getDummyImageBaseUrl() : $this->getImageBaseUrl();
+        $imageBaseUrl = $this->getImageBaseUrl();
         $allTemplates = $this->getAllTemplates();
         $actualCheckCommand = $this->obscuredCheckCommand === null ? $this->checkCommand : $this->obscuredCheckCommand;
         $concreteTemplates = $allTemplates->getTemplates($actualCheckCommand);
@@ -218,10 +246,12 @@ abstract class Graphs extends AbstractWidget
         IPT::recordf("Icinga check command: %s", $this->checkCommand);
         IPT::recordf("Obscured check command: %s", $this->obscuredCheckCommand);
 
-        foreach ([
+        foreach (
+            [
             ['template', $concreteTemplates, []],
             ['default_template', $allTemplates->getDefaultTemplates(), $excludedMetrics],
-        ] as $templateSet) {
+            ] as $templateSet
+        ) {
             list($urlParam, $templates, $excludeMetrics) = $templateSet;
 
             if ($urlParam === 'template') {
@@ -233,30 +263,24 @@ abstract class Graphs extends AbstractWidget
             IPT::indent();
 
             foreach ($templates as $templateName => $template) {
-                if ($this->designedForMyMonitoredObjectType($template)) {
+                if ($this->designedForObjectType($template)) {
                     IPT::recordf('Applying template %s', $templateName);
                     IPT::indent();
 
                     $charts = $template->getCharts(
-                        static::getMetricsDataSource(), $this->monitoredObject, [], $excludeMetrics
+                        static::getMetricsDataSource(),
+                        $this->object,
+                        [],
+                        $excludeMetrics
                     );
 
                     if (! empty($charts)) {
                         $currentGraphs = [];
 
                         foreach ($charts as $chart) {
-                            $metricVariables = $chart->getMetricVariables();
-                            $imageUrl = $this->filterImageUrl($imageBaseUrl->with($metricVariables))
-                                ->setParam($urlParam, $templateName)
-                                ->setParam('start', $this->start)
-                                ->setParam('end', $this->end)
-                                ->setParam('width', $this->width)
-                                ->setParam('height', $this->height)
-                                ->setParam('cachebuster', time() * 65536 + mt_rand(0, 65535));
-                            if (! $this->compact) {
-                                $imageUrl->setParam('legend', 1);
-                            }
+                            /** @var Chart $chart */
 
+                            $metricVariables = $chart->getMetricVariables();
                             $bestIntersect = -1;
                             $bestPos = count($result);
 
@@ -270,15 +294,61 @@ abstract class Graphs extends AbstractWidget
                             }
                             unset($graph);
 
-                            $currentGraphs[] = [
-                                '<img id="graphiteImg-'
-                                . md5((string) $imageUrl->without('cachebuster'))
-                                . "\" src=\"$imageUrl\" class=\"detach graphiteImg\" alt=\"\""
-                                . " width=\"$this->width\" height=\"$this->height\""
-                                . " style=\"min-width: {$this->width}px; min-height: {$this->height}px;\">",
-                                $metricVariables,
-                                $bestPos
-                            ];
+                            $urlParams = $template->getUrlParams();
+                            if (array_key_exists("height", $urlParams)) {
+                                $actheight = $urlParams["height"]->resolve(['height']);
+                                if ($actheight < $this->height) {
+                                    $actheight = $this->height;
+                                }
+                            } else {
+                                $actheight = $this->height;
+                            }
+                            $actwidth = $this->width;
+                            $actwidthfix = "";
+                            if (array_key_exists("width", $urlParams)) {
+                                $actwidth = $urlParams["width"]->resolve(['width']);
+                                $actwidthfix = "width: {$actwidth}px; ";
+                            }
+
+                            if ($this->renderInline) {
+                                $chart->setFrom($this->start)
+                                    ->setUntil($this->end)
+                                    ->setWidth($actwidth)
+                                    ->setHeight($actheight)
+                                    ->setBackgroundColor('white')
+                                    ->setForegroundColor('black')
+                                    ->setMajorGridLineColor('grey')
+                                    ->setMinorGridLineColor('white')
+                                    ->setShowLegend(! $this->compact);
+
+                                $img = new InlineGraphImage($chart);
+                            } else {
+                                $imageUrl = $this->filterImageUrl($imageBaseUrl->with($metricVariables))
+                                    ->setParam($urlParam, $templateName)
+                                    ->setParam('start', $this->start)
+                                    ->setParam('end', $this->end)
+                                    ->setParam('width', $actwidth)
+                                    ->setParam('height', $actheight);
+
+                                if (! $this->compact) {
+                                    $imageUrl->setParam('legend', 1);
+                                }
+
+                                if ($this->preloadDummy) {
+                                    $src = 'data:image/png;base64,' // 1x1 dummy
+                                        . 'iVBORw0KGgoAAAANSUhEUgAAAAEAAAABAQMAAAAl21bKAAAAA1BMVEUAAACnej3aAAAAAXRS'
+                                        . 'TlMAQObYZgAAAApJREFUeAFjZAAAAAQAAhq+CAMAAAAASUVORK5CYII=';
+                                } else {
+                                    $src = $imageUrl;
+                                }
+
+                                $img = '<img id="graphiteImg-' . md5((string) $imageUrl) . '"'
+                                    . " src=\"$src\" data-actualimageurl=\"$imageUrl\" class=\"detach graphiteImg\""
+                                    . " alt=\"\" width=\"$actwidth\" height=\"$actheight\""
+                                    . " style=\"min-width: {$actwidth}px; $actwidthfix min-height: {$actheight}px;\">";
+                            }
+
+                            $currentGraphs[] = [$img, $metricVariables, $bestPos];
                         }
 
                         foreach (array_reverse($currentGraphs) as $graph) {
@@ -303,7 +373,7 @@ abstract class Graphs extends AbstractWidget
             unset($graph);
 
             $currentUrl = Icinga::app()->getRequest()->getUrl();
-            $limit = (int) $currentUrl->getParam('graphs_limit', 50);
+            $limit = (int) $currentUrl->getParam('graphs_limit', 2);
             $total = count($result);
 
             if ($limit < 1) {
@@ -318,7 +388,7 @@ abstract class Graphs extends AbstractWidget
                     $view = $this->view();
 
                     $url = $this->getGraphsListBaseUrl();
-                    TimeRangePickerTrait::copyAllRangeParameters($url->getParams(), $currentUrl->getParams());
+                    TimeRangePickerTools::copyAllRangeParameters($url->getParams(), $currentUrl->getParams());
 
                     $result[] = "<p class='load-more'>{$view->qlink(
                         sprintf($view->translate('Load all %d graphs'), $total),
@@ -329,25 +399,6 @@ abstract class Graphs extends AbstractWidget
                 }
             }
 
-            if ($this->maxVisibleGraphs && count($result) > $this->maxVisibleGraphs) {
-                /** @var View $view */
-                $view = $this->view();
-
-                array_splice($result, $this->maxVisibleGraphs, 0, [sprintf(
-                    '<input type="checkbox" id="toggle-%1$s" class="expandable-toggle">'
-                        . '<label for="toggle-%1$s" class="link-button">'
-                        . '<span class="expandable-expand-label">%2$s</span>'
-                        . '<span class="expandable-collapse-label">%3$s</span>'
-                        . '</label>'
-                        . '<div class="expandable-content">',
-                    $view->protectId($this->getMonitoredObjectIdentifier()),
-                    $view->translate('Show More'),
-                    $view->translate('Show Less')
-                )]);
-
-                $result[] = '</div>';
-            }
-
             $classes = $this->classes;
             $classes[] = 'images';
 
@@ -355,6 +406,15 @@ abstract class Graphs extends AbstractWidget
             $result[] = '</div>';
         }
 
+        if ($this->renderInline) {
+            foreach ($result as $html) {
+                if ($html instanceof InlineGraphImage) {
+                    // Errors should occur now or not at all
+                    $html->render();
+                }
+            }
+        }
+
         return implode($result);
     }
 
@@ -403,15 +463,15 @@ abstract class Graphs extends AbstractWidget
     protected function getRangeFromTimeRangePicker(Request $request)
     {
         $params = $request->getUrl()->getParams();
-        $relative = $params->get(TimeRangePickerTrait::getRelativeRangeParameter());
+        $relative = $params->get(TimeRangePickerTools::getRelativeRangeParameter());
         if ($relative !== null) {
             return ["-$relative", null];
         }
 
-        $absolute = TimeRangePickerTrait::getAbsoluteRangeParameters();
+        $absolute = TimeRangePickerTools::getAbsoluteRangeParameters();
         $start = $params->get($absolute['start']);
         return [
-            $start === null ? -TimeRangePickerTrait::getDefaultRelativeTimeRange() : $start,
+            $start === null ? -TimeRangePickerTools::getDefaultRelativeTimeRange() : $start,
             $params->get($absolute['end'])
         ];
     }
@@ -430,13 +490,6 @@ abstract class Graphs extends AbstractWidget
      */
     abstract protected function getImageBaseUrl();
 
-    /**
-     * Get the base URL to a dummy image specifying just the monitored object kind
-     *
-     * @return Url
-     */
-    abstract protected function getDummyImageBaseUrl();
-
     /**
      * Get the base URL to the monitored object's graphs list
      *
@@ -454,13 +507,13 @@ abstract class Graphs extends AbstractWidget
     abstract protected function filterImageUrl(Url $url);
 
     /**
-     * Return whether the given template is designed for the type of the monitored object we display graphs for
+     * Return whether the given template is designed for the type of the object we display graphs for
      *
      * @param   Template    $template
      *
      * @return  bool
      */
-    abstract protected function designedForMyMonitoredObjectType(Template $template);
+    abstract protected function designedForObjectType(Template $template);
 
     /**
      * Get {@link compact}
@@ -557,29 +610,6 @@ abstract class Graphs extends AbstractWidget
         return $this;
     }
 
-    /**
-     * Get the amount of graphs to show
-     *
-     * @return  int
-     */
-    public function getMaxVisbileGraphs()
-    {
-        return $this->maxVisibleGraphs;
-    }
-
-    /**
-     * Set the amount of graphs to show
-     *
-     * @param   int     $count
-     *
-     * @return  $this
-     */
-    public function setMaxVisibleGraphs($count)
-    {
-        $this->maxVisibleGraphs = (int) $count;
-        return $this;
-    }
-
     /**
      * Get whether to serve a transparent dummy image first and let the JS code load the actual graph
      *
@@ -604,6 +634,30 @@ abstract class Graphs extends AbstractWidget
         return $this;
     }
 
+    /**
+     * Get whether to render the graphs inline
+     *
+     * @return bool
+     */
+    public function getRenderInline()
+    {
+        return $this->renderInline;
+    }
+
+    /**
+     * Set whether to render the graphs inline
+     *
+     * @param bool $renderInline
+     *
+     * @return $this
+     */
+    public function setRenderInline($renderInline = true)
+    {
+        $this->renderInline = $renderInline;
+
+        return $this;
+    }
+
     /**
      * Get whether to explicitly display that no graphs were found
      *
